@@ -1096,7 +1096,7 @@ QString KUrl::prettyUrl( AdjustPathOption trailing ) const
   QString result = scheme();
   if (!result.isEmpty())
   {
-    if(!authority().isEmpty() || result == QLatin1String("file"))
+    if (!authority().isEmpty() || result == QLatin1String("file") || path().isEmpty())
         result += QLatin1String("://");
     else
         result += QLatin1Char(':');
