@@ -1102,8 +1102,7 @@ QStringList KStandardDirs::KStandardDirsPrivate::resourceDirs(const char* type,
                 restrictionActive = true;
         }
 
-        QStringList dirs;
-        dirs = m_relatives.value(type);
+        const QStringList dirs = m_relatives.value(type);
         const QString typeInstallPath = installPath(type); // could be empty
 // better #ifdef incasesensitive_filesystem
 #ifdef Q_WS_WIN
@@ -1197,23 +1196,22 @@ QStringList KStandardDirs::KStandardDirsPrivate::resourceDirs(const char* type,
                 candidates.append(installdir);
         }
 
-        dirs = m_absolutes.value(type);
-        if (!dirs.isEmpty())
-            for (QStringList::ConstIterator it = dirs.constBegin();
-                 it != dirs.constEnd(); ++it)
-            {
-                testdir.setPath(*it);
-                if (testdir.exists()) {
+        const QStringList absDirs = m_absolutes.value(type);
+        for (QStringList::ConstIterator it = absDirs.constBegin();
+             it != absDirs.constEnd(); ++it)
+        {
+            testdir.setPath(*it);
+            if (testdir.exists()) {
 #ifdef Q_WS_WIN
-                    const QString filename = realPath( *it ).toLower();
+                const QString filename = realPath( *it ).toLower();
 #else
-                    const QString filename = realPath( *it );
+                const QString filename = realPath( *it );
 #endif
-                    if (!candidates.contains(filename)) {
-                        candidates.append(filename);
-                    }
+                if (!candidates.contains(filename)) {
+                    candidates.append(filename);
                 }
             }
+        }
 
         // Insert result into the cache for next time.
         // Exception: data_subdir restrictions are per-subdir, so we can't store such results
@@ -1866,6 +1864,8 @@ void KStandardDirs::addKDEDefaults()
 
     addResourceType("lib", 0, "lib" KDELIBSUFF "/");
 
+    addResourceType("qtplugins", "lib", "plugins");
+
     uint index = 0;
     while (types_indices[index] != -1) {
         addResourceType(types_string + types_indices[index], 0, types_string + types_indices[index+1], true);
