@@ -61,8 +61,6 @@ public:
     void updateLayout();
     void slotTimeLineChanged(qreal);
     void slotTimeLineFinished();
-
-    int bestContentHeight() const;
 };
 
 void KMessageWidgetPrivate::init(KMessageWidget *q_ptr)
@@ -86,6 +84,7 @@ void KMessageWidgetPrivate::init(KMessageWidget *q_ptr)
     textLabel = new QLabel(content);
     textLabel->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
     textLabel->setTextInteractionFlags(Qt::TextBrowserInteraction);
+    QObject::connect(textLabel, SIGNAL(linkActivated(const QString&)), q, SIGNAL(linkActivated(const QString&)));
 
     KAction* closeAction = KStandardAction::close(q, SLOT(animatedHide()), q);
 
@@ -161,10 +160,6 @@ void KMessageWidgetPrivate::updateLayout()
 
 void KMessageWidgetPrivate::updateSnapShot()
 {
-    // Attention: updateSnapShot calls QWidget::render(), which causes the whole
-    // window layouts to be activated. Calling this method from resizeEvent()
-    // can lead to infinite recursion, see:
-    // https://bugs.kde.org/show_bug.cgi?id=311336
     contentSnapShot = QPixmap(content->size());
     contentSnapShot.fill(Qt::transparent);
     content->render(&contentSnapShot, QPoint(), QRegion(), QWidget::DrawChildren);
@@ -180,24 +175,13 @@ void KMessageWidgetPrivate::slotTimeLineFinished()
 {
     if (timeLine->direction() == QTimeLine::Forward) {
         // Show
-        // We set the whole geometry here, because it may be wrong if a
-        // KMessageWidget is shown right when the toplevel window is created.
-        content->setGeometry(0, 0, q->width(), bestContentHeight());
+        content->move(0, 0);
     } else {
         // Hide
         q->hide();
     }
 }
 
-int KMessageWidgetPrivate::bestContentHeight() const
-{
-    int height = content->heightForWidth(q->width());
-    if (height == -1) {
-        height = content->sizeHint().height();
-    }
-    return height;
-}
-
 
 //---------------------------------------------------------------------
 // KMessageWidget
@@ -323,9 +307,12 @@ bool KMessageWidget::event(QEvent* event)
 void KMessageWidget::resizeEvent(QResizeEvent* event)
 {
     QFrame::resizeEvent(event);
-
     if (d->timeLine->state() == QTimeLine::NotRunning) {
-        d->content->resize(width(), d->bestContentHeight());
+        int contentHeight = d->content->heightForWidth(width());
+        if (contentHeight == -1) {
+            contentHeight = d->content->sizeHint().height();
+        }
+        d->content->resize(width(), contentHeight);
     }
 }
 
@@ -409,7 +396,7 @@ void KMessageWidget::animatedShow()
 
     QFrame::show();
     setFixedHeight(0);
-    int wantedHeight = d->bestContentHeight();
+    int wantedHeight = d->content->sizeHint().height();
     d->content->setGeometry(0, -wantedHeight, width(), wantedHeight);
 
     d->updateSnapShot();
