@@ -28,7 +28,6 @@
 #include <dom/dom_exception.h>
 #include <css/cssstyleselector.h>
 
-#include <misc/htmlhashes.h>
 #include <rendering/render_text.h>
 #include <rendering/render_flow.h>
 #include <wtf/PassRefPtr.h>
@@ -70,7 +69,9 @@ void CharacterDataImpl::setData( const DOMString &_data, int &exceptioncode )
     if(str == _data.impl) return; // ### fire DOMCharacterDataModified if modified?
     DOMStringImpl *oldStr = str;
     str = _data.impl;
-    if(str) str->ref();
+    if (!str)
+        str = new DOMStringImpl((QChar*)0, 0);
+    str->ref();
     if (m_render)
       (static_cast<RenderText*>(m_render))->setText(str);
     setChanged(true);
@@ -312,7 +313,7 @@ bool CommentImpl::childTypeAllowed( unsigned short /*type*/ )
 DOMString CommentImpl::toString() const
 {
     // FIXME: substitute entity references as needed!
-    return DOMString("<!--") + escapeHTML( nodeValue() ) + "-->";
+    return DOMString("<!--") + nodeValue() + DOMString("-->");
 }
 
 // ---------------------------------------------------------------------------
@@ -412,6 +413,7 @@ DOMString TextImpl::wholeText() const
 
 TextImpl* TextImpl::replaceWholeText(const DOMString& newText, int &ec)
 {
+    Q_UNUSED(ec);
     // We don't support "read-only" text nodes (no Entity node support)
     // Thus, we remove all adjacent text nodes, and replace the contents of this one.
     assert(!isReadOnly());
@@ -473,17 +475,17 @@ bool TextImpl::rendererIsNeeded(RenderStyle *style)
     if (!onlyWS) {
         return true;
     }
-    
+
     RenderObject *par = parentNode()->renderer();
-    
+
     if (par->isTable() || par->isTableRow() || par->isTableSection()) {
         return false;
     }
-    
+
     if (style->preserveWS() || style->preserveLF()) {
         return true;
     }
-    
+
     RenderObject *prev = previousRenderer();
     if (par->isInlineFlow()) {
         // <span><div/> <div/></span>
@@ -494,7 +496,7 @@ bool TextImpl::rendererIsNeeded(RenderStyle *style)
         if (par->isRenderBlock() && !par->childrenInline() && (!prev || !prev->isInline())) {
             return false;
         }
-        
+
         RenderObject *first = par->firstChild();
         while (first && first->isFloatingOrPositioned())
             first = first->nextSibling();
@@ -505,7 +507,7 @@ bool TextImpl::rendererIsNeeded(RenderStyle *style)
             return false;
         }
     }
-    
+
     return true;
 }
 
@@ -558,9 +560,9 @@ TextImpl *TextImpl::createNew(DOMStringImpl *_str)
 
 DOMStringImpl* TextImpl::renderString() const
 {
-    if (renderer()) 
+    if (renderer())
         return static_cast<RenderText*>(renderer())->string();
-    else 
+    else
         return string();
 }
 
@@ -598,7 +600,7 @@ DOMString TextImpl::toString(long long startOffset, long long endOffset) const
     if(endOffset >=0 || startOffset >0)
 	str = str.copy(); //we are going to modify this, so make a copy.  I hope I'm doing this right.
     if(endOffset >= 0)
-        str.truncate(endOffset); 
+        str.truncate(endOffset);
     if(startOffset > 0)    //note the order of these 2 'if' statements so that it works right when n==m_startContainer==m_endContainer
         str.remove(0, startOffset);
     return textNeedsEscaping( this ) ? escapeHTML( str ) : str;
@@ -634,11 +636,11 @@ TextImpl *CDATASectionImpl::createNew(DOMStringImpl *_str)
 
 DOMString CDATASectionImpl::toString() const
 {
-    return DOMString("<![CDATA[") + nodeValue() + "]]>";
+    return DOMString("<![CDATA[") + nodeValue() + DOMString("]]>");
 }
 
 // ---------------------------------------------------------------------------
- 
+
 EditingTextImpl::EditingTextImpl(DocumentImpl *impl, const DOMString &text)
     : TextImpl(impl, text.implementation())
 {
@@ -658,5 +660,4 @@ bool EditingTextImpl::rendererIsNeeded(RenderStyle */*style*/)
     return true;
 }
 
-  
-
+// kate: indent-width 4; replace-tabs on; tab-width 8; space-indent on;
