@@ -17,76 +17,187 @@
     Boston, MA 02110-1301, USA.
 */
 
-
 #include "httpauthentication.h"
 
+#ifdef HAVE_LIBGSSAPI
+#ifdef GSSAPI_MIT
+#include <gssapi/gssapi.h>
+#else
+#include <gssapi.h>
+#endif /* GSSAPI_MIT */
+
+// Catch uncompatible crap (BR86019)
+#if defined(GSS_RFC_COMPLIANT_OIDS) && (GSS_RFC_COMPLIANT_OIDS == 0)
+#include <gssapi/gssapi_generic.h>
+#define GSS_C_NT_HOSTBASED_SERVICE gss_nt_service_name
+#endif
+
+#endif /* HAVE_LIBGSSAPI */
+
+#include <krandom.h>
+#include <kdebug.h>
+#include <klocale.h>
+#include <kglobal.h>
+#include <kcodecs.h>
+#include <kconfiggroup.h>
+#include <kio/authinfo.h>
+#include <misc/kntlm/kntlm.h>
+
+#include <QtCore/QTextCodec>
+
+
+static bool isWhiteSpace(char ch)
+{
+    return (ch == ' ' || ch == '\t' || ch == '\v' || ch == '\f');
+}
+
+static bool isWhiteSpaceOrComma(char ch)
+{
+    return (ch == ',' || isWhiteSpace(ch));
+}
+
+static bool containsScheme(const char input[], int start, int end)
+{
+    // skip any comma or white space
+    while (start < end && isWhiteSpaceOrComma(input[start])) {
+        start++;
+    }
+
+    while (start < end) {
+        if (isWhiteSpace(input[start])) {
+            return true;
+        }
+        start++;
+    }
+
+    return false;
+}
 
 // keys on even indexes, values on odd indexes. Reduces code expansion for the templated
 // alternatives.
-static QList<QByteArray> parseChallenge(const QByteArray &ba, QByteArray *scheme)
+// If "ba" starts with empty content it will be removed from ba to simplify later calls
+static QList<QByteArray> parseChallenge(QByteArray &ba, QByteArray *scheme, QByteArray* nextAuth = 0)
 {
     QList<QByteArray> values;
-    const int len = ba.count();
     const char *b = ba.constData();
-    int start = 0;
-    int end = 0;
+    int len = ba.count();
+    int start = 0, end = 0, pos = 0, pos2 = 0;
 
     // parse scheme
-    while (start < len && (b[start] == ' ' || b[start] == '\t')) {
+    while (start < len && isWhiteSpaceOrComma(b[start])) {
         start++;
     }
     end = start;
-    while (end < len && (b[end] != ' ' && b[end] != '\t')) {
+    while (end < len && !isWhiteSpace(b[end])) {
         end++;
     }
-    if (scheme) {
-        *scheme = QByteArray(b + start, end - start);
+
+    // drop empty stuff from the given string, it would have to be skipped over and over again
+    if (start != 0) {
+        ba = ba.mid(start);
+        end -= start;
+        len -= start;
+        start = 0;
+        b = ba.constData();
     }
+    Q_ASSERT(scheme);
+    *scheme = ba.left(end);
 
     while (end < len) {
         start = end;
-        // parse key
-        while (start < len && (b[start] == ' ' || b[start] == '\t')) {
-            start++;
-        }
-        end = start;
         while (end < len && b[end] != '=') {
             end++;
         }
-        values.append(QByteArray(b + start, end - start));
+        pos = end; // save the end position
+        while (end - 1 > start && isWhiteSpace(b[end - 1])) { // trim whitespace
+            end--;
+        }
+        pos2 = start;
+        while (pos2 < end && isWhiteSpace(b[pos2])) { // skip whitespace
+            pos2++;
+        }
+        if (containsScheme(b, start, end) || (b[pos2] == ',' && b[pos] != '=' && pos == len)) {
+            if (nextAuth) {
+                *nextAuth = QByteArray (b + start);
+            }
+            break;  // break on start of next scheme.
+        }
+        while (start < len && isWhiteSpaceOrComma(b[start])) {
+            start++;
+        }
+        values.append(QByteArray (b + start, end - start));
+        end = pos; // restore the end position
         if (end == len) {
             break;
         }
 
         // parse value
         start = end + 1;    //skip '='
-        while (start < len && (b[start] == ' ' || b[start] == '\t')) {
+        while (start < len && isWhiteSpace(b[start])) {
             start++;
         }
-        if (start + 1 < len && b[start] == '"') {
-            end = ++start;
+
+        if (b[start] == '"') {
             //quoted string
-            while (end < len && b[end] != '"') {
-                end++;
+            bool hasBs = false;
+            bool hasErr = false;
+            end = ++start;
+            while (end < len) {
+                if (b[end] == '\\') {
+                    end++;
+                    if (end + 1 >= len) {
+                        hasErr = true;
+                        break;
+                    } else {
+                        hasBs = true;
+                        end++;
+                    }
+                } else if (b[end] == '"') {
+                    break;
+                } else {
+                    end++;
+                }
             }
-            values.append(QByteArray(b + start, end - start));
-            //the quoted string has ended, but only a comma ends a key-value pair
-            while (end < len && b[end] != ',') {
-                end++;
+            if (hasErr || (end == len)) {
+                // remove the key we already inserted
+                kDebug(7113) << "error in quoted text for key" << values.last();
+                values.removeLast();
+                break;
+             }
+            QByteArray value = QByteArray(b + start, end - start);
+            if (hasBs) {
+                // skip over the next character, it might be an escaped backslash
+                int i = -1;
+                while ( (i = value.indexOf('\\', i + 1)) >= 0 ) {
+                    value.remove(i, 1);
+                }
             }
+            values.append(value);
+            end++;
         } else {
-            end = start;
             //unquoted string
-            while (end < len && b[end] != ',') {
+            end = start;
+            while (end < len && b[end] != ',' && !isWhiteSpace(b[end])) {
                 end++;
             }
             values.append(QByteArray(b + start, end - start));
         }
-        // end may point beyond the buffer already here
-        end++;  // skip comma
+
+        //the quoted string has ended, but only a comma ends a key-value pair
+        while (end < len && isWhiteSpace(b[end])) {
+            end++;
+        }
+
+        // garbage, here should be end or field delimiter (comma)
+        if (end < len && b[end] != ',') {
+            kDebug(7113) << "unexpected character" << b[end] << "found in WWW-authentication header where token boundary (,) was expected";
+            break;
+        }
     }
     // ensure every key has a value
-    if (values.count() % 2) {
+    // WARNING: Do not remove the > 1 check or parsing a Type 1 NTLM
+    // authentication challenge will surely fail.
+    if (values.count() > 1 && values.count() % 2) {
         values.removeLast();
     }
     return values;
@@ -95,7 +206,7 @@ static QList<QByteArray> parseChallenge(const QByteArray &ba, QByteArray *scheme
 
 static QByteArray valueForKey(const QList<QByteArray> &ba, const QByteArray &key)
 {
-    for (int i = 0; i + 1 < ba.count(); i += 2) {
+    for (int i = 0, count = ba.count(); (i + 1) < count; i += 2) {
         if (ba[i] == key) {
             return ba[i + 1];
         }
@@ -103,6 +214,15 @@ static QByteArray valueForKey(const QList<QByteArray> &ba, const QByteArray &key
     return QByteArray();
 }
 
+KAbstractHttpAuthentication::KAbstractHttpAuthentication(KConfigGroup *config)
+                            :m_config(config), m_finalAuthStage(false)
+{
+    reset();
+}
+
+KAbstractHttpAuthentication::~KAbstractHttpAuthentication()
+{
+}
 
 QByteArray KAbstractHttpAuthentication::bestOffer(const QList<QByteArray> &offers)
 {
@@ -111,104 +231,132 @@ QByteArray KAbstractHttpAuthentication::bestOffer(const QList<QByteArray> &offer
     QByteArray digestOffer;
     QByteArray ntlmOffer;
     QByteArray basicOffer;
-    foreach (const QByteArray &offer, offers) {
-        QByteArray scheme = offer.mid(0, 10).toLower();
+    Q_FOREACH (const QByteArray &offer, offers) {
+        const QByteArray scheme = offer.mid(0, offer.indexOf(' ')).toLower();
 #ifdef HAVE_LIBGSSAPI
-        if (scheme.startsWith("negotiate")) {
+        if (scheme == "negotiate") { // krazy:exclude=strings
             negotiateOffer = offer;
         } else
 #endif
-        if (scheme.startsWith("digest")) {
+        if (scheme == "digest") { // krazy:exclude=strings
             digestOffer = offer;
-        } else if (scheme.startsWith("ntlm")) {
+        } else if (scheme == "ntlm") { // krazy:exclude=strings
             ntlmOffer = offer;
-        } else if (scheme.startsWith("basic")) {
+        } else if (scheme == "basic") { // krazy:exclude=strings
             basicOffer = offer;
         }
     }
 
     if (!negotiateOffer.isEmpty()) {
         return negotiateOffer;
-    } else if (!digestOffer.isEmpty()) {
+    }
+
+    if (!digestOffer.isEmpty()) {
         return digestOffer;
-    } else if (!ntlmOffer.isEmpty()) {
+    }
+
+    if (!ntlmOffer.isEmpty()) {
         return ntlmOffer;
     }
+
     return basicOffer;  //empty or not...
 }
 
 
-KAbstractHttpAuthentication *KAbstractHttpAuthentication::newAuth(const QByteArray &offer)
+KAbstractHttpAuthentication *KAbstractHttpAuthentication::newAuth(const QByteArray &offer, KConfigGroup* config)
 {
-    QByteArray scheme = offer.mid(0, 10).toLower();
+    const QByteArray scheme = offer.mid(0, offer.indexOf(' ')).toLower();
 #ifdef HAVE_LIBGSSAPI
-    if (scheme.startsWith("negotiate")) {
-        return new KHttpNegotiateAuthentication();
+    if (scheme == "negotiate") { // krazy:exclude=strings
+        return new KHttpNegotiateAuthentication(config);
     } else
 #endif
-    if (scheme.startsWith("digest")) {
+    if (scheme == "digest") { // krazy:exclude=strings
         return new KHttpDigestAuthentication();
-    } else if (scheme.startsWith("ntlm")) {
-        return new KHttpNtlmAuthentication();
-    } else if (scheme.startsWith("basic")) {
+    } else if (scheme == "ntlm") { // krazy:exclude=strings
+        return new KHttpNtlmAuthentication(config);
+    } else if (scheme == "basic") { // krazy:exclude=strings
         return new KHttpBasicAuthentication();
     }
     return 0;
 }
 
+QList< QByteArray > KAbstractHttpAuthentication::splitOffers(const QList< QByteArray >& offers)
+{
+    // first detect if one entry may contain multiple offers
+    QList<QByteArray> alloffers;
+    foreach(QByteArray offer, offers) {
+        QByteArray scheme, cont;
+
+        parseChallenge(offer, &scheme, &cont);
+
+        while (!cont.isEmpty()) {
+            offer.chop(cont.length());
+            alloffers << offer;
+            offer = cont;
+            cont.clear();
+            parseChallenge(offer, &scheme, &cont);
+        }
+        alloffers << offer;
+    }
+    return alloffers;
+}
 
 void KAbstractHttpAuthentication::reset()
 {
     m_scheme.clear();
     m_challenge.clear();
+    m_challengeText.clear();
     m_resource.clear();
     m_httpMethod.clear();
     m_isError = false;
     m_needCredentials = true;
     m_forceKeepAlive = false;
     m_forceDisconnect = false;
+    m_keepPassword = false;
     m_headerFragment.clear();
     m_username.clear();
     m_password.clear();
 }
 
-
 void KAbstractHttpAuthentication::setChallenge(const QByteArray &c, const KUrl &resource,
                                                const QByteArray &httpMethod)
 {
     reset();
-    m_challenge = parseChallenge(c, &m_scheme);
+    m_challengeText = c.trimmed();
+    m_challenge = parseChallenge(m_challengeText, &m_scheme);
     Q_ASSERT(m_scheme.toLower() == scheme().toLower());
     m_resource = resource;
-    m_httpMethod = httpMethod;
+    m_httpMethod = httpMethod.trimmed();
 }
 
 
 QString KAbstractHttpAuthentication::realm() const
 {
-    QByteArray realm = valueForKey(m_challenge, "realm");
-    if (KGlobal::locale()->language().contains("ru")) {
+    const QByteArray realm = valueForKey(m_challenge, "realm");
+    // TODO: Find out what this is supposed to address. The site mentioned below does not exist.
+    if (KGlobal::locale()->language().contains(QLatin1String("ru"))) {
         //for sites like lib.homelinux.org
         return QTextCodec::codecForName("CP1251")->toUnicode(realm);
     }
-    return QString::fromLatin1(realm);
+    return QString::fromLatin1(realm.constData(), realm.length());
 }
 
-
 void KAbstractHttpAuthentication::authInfoBoilerplate(KIO::AuthInfo *a) const
 {
-    a->verifyPath = true;  //### research this
     a->url = m_resource;
-    a->realmValue = realm();
     a->username = m_username;
     a->password = m_password;
+    a->verifyPath = supportsPathMatching();
+    a->realmValue = realm();
+    a->digestInfo = QLatin1String(authDataToCache());
+    a->keepPassword = m_keepPassword;
 }
 
 
 void KAbstractHttpAuthentication::generateResponseCommon(const QString &user, const QString &password)
 {
-    if ((m_needCredentials && (user.isEmpty() || password.isEmpty())) ||
-        m_scheme.isEmpty() || m_httpMethod.isEmpty()) {
+    if (m_scheme.isEmpty() || m_httpMethod.isEmpty()) {
         m_isError = true;
         return;
     }
@@ -217,11 +365,11 @@ void KAbstractHttpAuthentication::generateResponseCommon(const QString &user, co
         m_username = user;
         m_password = password;
     }
+
     m_isError = false;
-    // we could leave m_needCredentials value alone; this is just defensive coding.
-    m_needCredentials = true;
     m_forceKeepAlive = false;
     m_forceDisconnect = false;
+    m_finalAuthStage = true;
 }
 
 
@@ -236,7 +384,6 @@ void KHttpBasicAuthentication::fillKioAuthInfo(KIO::AuthInfo *ai) const
     authInfoBoilerplate(ai);
 }
 
-
 void KHttpBasicAuthentication::generateResponse(const QString &user, const QString &password)
 {
     generateResponseCommon(user, password);
@@ -245,9 +392,8 @@ void KHttpBasicAuthentication::generateResponse(const QString &user, const QStri
     }
 
     m_headerFragment = "Basic ";
-    m_headerFragment += KCodecs::base64Encode(m_username.toLatin1() + ':' + m_password.toLatin1());
+    m_headerFragment += QByteArray(m_username.toLatin1() + ':' + m_password.toLatin1()).toBase64();
     m_headerFragment += "\r\n";
-    return;
 }
 
 
@@ -303,12 +449,6 @@ struct DigestAuthInfo
 //calculateResponse() from the original HTTPProtocol
 static QByteArray calculateResponse(const DigestAuthInfo &info, const KUrl &resource)
 {
-  kDebug(7113) << info.nc << info.qop << info.realm << info.nonce << info.method << info.cnonce
-               << info.username << info.password << info.algorithm << info.entityBody;
-  foreach (const KUrl &u, info.digestURIs) {
-      kDebug(7113) << u;
-  }
-  kDebug(7113);
   KMD5 md;
   QByteArray HA1;
   QByteArray HA2;
@@ -342,7 +482,9 @@ static QByteArray calculateResponse(const DigestAuthInfo &info, const KUrl &reso
   if ( info.qop == "auth-int" )
   {
     authStr += ':';
-    authStr += info.entityBody;
+    md.reset();
+    md.update(info.entityBody);
+    authStr += md.hexDigest();
   }
   md.reset();
   md.update( authStr );
@@ -368,10 +510,9 @@ static QByteArray calculateResponse(const DigestAuthInfo &info, const KUrl &reso
   md.reset();
   md.update( authStr );
 
-  QByteArray Response = md.hexDigest();
-
-  kDebug(7113) << "Response => " << Response;
-  return Response;
+  const QByteArray response = md.hexDigest();
+  kDebug(7113) << "Response =>" << response;
+  return response;
 }
 
 
@@ -382,7 +523,7 @@ void KHttpDigestAuthentication::generateResponse(const QString &user, const QStr
         return;
     }
 
-// magic starts here (this part is slightly modified from the original in HTTPProtocol)
+    // magic starts here (this part is slightly modified from the original in HTTPProtocol)
 
     DigestAuthInfo info;
 
@@ -395,7 +536,11 @@ void KHttpDigestAuthentication::generateResponse(const QString &user, const QStr
     info.qop = "";
 
     // cnonce is recommended to contain about 64 bits of entropy
+#ifdef ENABLE_HTTP_AUTH_NONCE_SETTER
+    info.cnonce = m_nonce;
+#else
     info.cnonce = KRandom::randomString(16).toLatin1();
+#endif
 
     // HACK: Should be fixed according to RFC 2617 section 3.2.2
     info.nc = "00000001";
@@ -406,15 +551,15 @@ void KHttpDigestAuthentication::generateResponse(const QString &user, const QStr
     // Parse the Digest response....
     info.realm = valueForKey(m_challenge, "realm");
 
-    info.algorithm = valueForKey(m_challenge, "algorith");
+    info.algorithm = valueForKey(m_challenge, "algorithm");
     if (info.algorithm.isEmpty()) {
-        info.algorithm = valueForKey(m_challenge, "algorithm");
+        info.algorithm = valueForKey(m_challenge, "algorith");
     }
     if (info.algorithm.isEmpty()) {
         info.algorithm = "MD5";
     }
 
-    foreach (const QByteArray &path, valueForKey(m_challenge, "domain").split(' ')) {
+    Q_FOREACH (const QByteArray &path, valueForKey(m_challenge, "domain").split(' ')) {
         KUrl u(m_resource, QString::fromLatin1(path));
         if (u.isValid()) {
             info.digestURIs.append(u);
@@ -425,108 +570,129 @@ void KHttpDigestAuthentication::generateResponse(const QString &user, const QStr
     QByteArray opaque = valueForKey(m_challenge, "opaque");
     info.qop = valueForKey(m_challenge, "qop");
 
+    // NOTE: Since we do not have access to the entity body, we cannot support
+    // the "auth-int" qop value ; so if the server returns a comma separated
+    // list of qop values, prefer "auth".See RFC 2617 sec 3.2.2 for the details.
+    // If "auth" is not present or it is set to "auth-int" only, then we simply
+    // print a warning message and disregard the qop option altogether.
+    if (info.qop.contains(',')) {
+        const QList<QByteArray> values = info.qop.split(',');
+        if (info.qop.contains("auth"))
+            info.qop = "auth";
+        else {
+            kWarning(7113) << "Unsupported digest authentication qop parameters:" << values;
+            info.qop.clear();
+        }
+    } else if (info.qop == "auth-int") {
+        kWarning(7113) << "Unsupported digest authentication qop parameter:" << info.qop;
+        info.qop.clear();
+    }
+
     if (info.realm.isEmpty() || info.nonce.isEmpty()) {
         // ### proper error return
         m_isError = true;
         return;
     }
 
-  // If the "domain" attribute was not specified and the current response code
-  // is authentication needed, add the current request url to the list over which
-  // this credential can be automatically applied.
-  if (info.digestURIs.isEmpty() /*###&& (m_request.responseCode == 401 || m_request.responseCode == 407)*/)
-    info.digestURIs.append (m_resource);
-  else
-  {
-    // Verify whether or not we should send a cached credential to the
-    // server based on the stored "domain" attribute...
-    bool send = true;
-
-    // Determine the path of the request url...
-    QString requestPath = m_resource.directory(KUrl::AppendTrailingSlash | KUrl::ObeyTrailingSlash);
-    if (requestPath.isEmpty())
-      requestPath = "/";
-
-    foreach (const KUrl &u, info.digestURIs)
+    // If the "domain" attribute was not specified and the current response code
+    // is authentication needed, add the current request url to the list over which
+    // this credential can be automatically applied.
+    if (info.digestURIs.isEmpty() /*###&& (m_request.responseCode == 401 || m_request.responseCode == 407)*/)
+        info.digestURIs.append (m_resource);
+    else
     {
-      send &= (m_resource.protocol().toLower() == u.protocol().toLower());
-      send &= (m_resource.host().toLower() == u.host().toLower());
+        // Verify whether or not we should send a cached credential to the
+        // server based on the stored "domain" attribute...
+        bool send = true;
 
-      if (m_resource.port() > 0 && u.port() > 0)
-        send &= (m_resource.port() == u.port());
+        // Determine the path of the request url...
+        QString requestPath = m_resource.directory(KUrl::AppendTrailingSlash | KUrl::ObeyTrailingSlash);
+        if (requestPath.isEmpty())
+          requestPath = QLatin1Char('/');
 
-      QString digestPath = u.directory (KUrl::AppendTrailingSlash | KUrl::ObeyTrailingSlash);
-      if (digestPath.isEmpty())
-        digestPath = "/";
+        Q_FOREACH (const KUrl &u, info.digestURIs)
+        {
+          send &= (m_resource.protocol().toLower() == u.protocol().toLower());
+          send &= (m_resource.host().toLower() == u.host().toLower());
 
-      send &= (requestPath.startsWith(digestPath));
+          if (m_resource.port() > 0 && u.port() > 0)
+            send &= (m_resource.port() == u.port());
 
-      if (send)
-        break;
+          QString digestPath = u.directory (KUrl::AppendTrailingSlash | KUrl::ObeyTrailingSlash);
+          if (digestPath.isEmpty())
+            digestPath = QLatin1Char('/');
+
+          send &= (requestPath.startsWith(digestPath));
+
+          if (send)
+            break;
+        }
+
+        if (!send) {
+            m_isError = true;
+            return;
+        }
     }
 
-    kDebug(7113) << "passed digest authentication credential test: " << send;
+    kDebug(7113) << "RESULT OF PARSING:";
+    kDebug(7113) << "  algorithm: " << info.algorithm;
+    kDebug(7113) << "  realm:     " << info.realm;
+    kDebug(7113) << "  nonce:     " << info.nonce;
+    kDebug(7113) << "  opaque:    " << opaque;
+    kDebug(7113) << "  qop:       " << info.qop;
 
-    if (!send) {
-        m_isError = true;
-        return;
+    // Calculate the response...
+    const QByteArray response = calculateResponse(info, m_resource);
+
+    QByteArray auth = "Digest username=\"";
+    auth += info.username;
+
+    auth += "\", realm=\"";
+    auth += info.realm;
+    auth += "\"";
+
+    auth += ", nonce=\"";
+    auth += info.nonce;
+
+    auth += "\", uri=\"";
+    auth += m_resource.encodedPathAndQuery(KUrl::LeaveTrailingSlash, KUrl::AvoidEmptyPath).toLatin1();
+
+    if (!info.algorithm.isEmpty()) {
+      auth += "\", algorithm=";
+      auth += info.algorithm;
     }
-  }
 
-  kDebug(7113) << "RESULT OF PARSING:";
-  kDebug(7113) << "  algorithm: " << info.algorithm;
-  kDebug(7113) << "  realm:     " << info.realm;
-  kDebug(7113) << "  nonce:     " << info.nonce;
-  kDebug(7113) << "  opaque:    " << opaque;
-  kDebug(7113) << "  qop:       " << info.qop;
+    if ( !info.qop.isEmpty() )
+    {
+      auth += ", qop=";
+      auth += info.qop;
+      auth += ", cnonce=\"";
+      auth += info.cnonce;
+      auth += "\", nc=";
+      auth += info.nc;
+    }
 
-  // Calculate the response...
-  QByteArray Response = calculateResponse(info, m_resource);
+    auth += ", response=\"";
+    auth += response;
+    if ( !opaque.isEmpty() )
+    {
+      auth += "\", opaque=\"";
+      auth += opaque;
+    }
+    auth += "\"\r\n";
 
-  QString auth = "Digest username=\"";
-  auth += info.username;
-
-  auth += "\", realm=\"";
-  auth += info.realm;
-  auth += "\"";
-
-  auth += ", nonce=\"";
-  auth += info.nonce;
-
-  auth += "\", uri=\"";
-  auth += m_resource.encodedPathAndQuery(KUrl::LeaveTrailingSlash, KUrl::AvoidEmptyPath);
-
-  if (!info.algorithm.isEmpty()) {
-    auth += "\", algorithm=\"";
-    auth += info.algorithm;
-    auth +="\"";
-  }
-
-  if ( !info.qop.isEmpty() )
-  {
-    auth += ", qop=\"";
-    auth += info.qop;
-    auth += "\", cnonce=\"";
-    auth += info.cnonce;
-    auth += "\", nc=";
-    auth += info.nc;
-  }
-
-  auth += ", response=\"";
-  auth += Response;
-  if ( !opaque.isEmpty() )
-  {
-    auth += "\", opaque=\"";
-    auth += opaque;
-  }
-  auth += "\"\r\n";
-
-// magic ends here
+    // magic ends here
     // note that auth already contains \r\n
     m_headerFragment = auth;
-    return;
 }
 
+#ifdef ENABLE_HTTP_AUTH_NONCE_SETTER
+void KHttpDigestAuthentication::setDigestNonceValue(const QByteArray& nonce)
+{
+    m_nonce = nonce;
+}
+#endif
+
 
 QByteArray KHttpNtlmAuthentication::scheme() const
 {
@@ -535,14 +701,21 @@ QByteArray KHttpNtlmAuthentication::scheme() const
 
 
 void KHttpNtlmAuthentication::setChallenge(const QByteArray &c, const KUrl &resource,
-                                             const QByteArray &httpMethod)
+                                           const QByteArray &httpMethod)
 {
-    KAbstractHttpAuthentication::setChallenge(c, resource, httpMethod);
-    if (m_challenge.isEmpty()) {
-        // The type 1 message we're going to send needs no credentials;
-        // they come later in the type 3 message.
-        m_needCredentials = false;
+    QString oldUsername, oldPassword;
+    if (!m_finalAuthStage && !m_username.isEmpty() && !m_password.isEmpty()) {
+        oldUsername = m_username;
+        oldPassword = m_password;
     }
+    KAbstractHttpAuthentication::setChallenge(c, resource, httpMethod);
+    if (!oldUsername.isEmpty() && !oldPassword.isEmpty()) {
+        m_username = oldUsername;
+        m_password = oldPassword;
+    }
+    // The type 1 message we're going to send needs no credentials;
+    // they come later in the type 3 message.
+    m_needCredentials = m_challenge.isEmpty();
 }
 
 
@@ -552,7 +725,7 @@ void KHttpNtlmAuthentication::fillKioAuthInfo(KIO::AuthInfo *ai) const
     // Every auth scheme is supposed to supply a realm according to the RFCs. Of course this doesn't
     // prevent Microsoft from not doing it... Dummy value!
     // we don't have the username yet which may (may!) contain a domain, so we really have no choice
-    ai->realmValue = "NTLM";
+    ai->realmValue = QLatin1String("NTLM");
 }
 
 
@@ -566,26 +739,41 @@ void KHttpNtlmAuthentication::generateResponse(const QString &_user, const QStri
     QByteArray buf;
 
     if (m_challenge.isEmpty()) {
+        m_finalAuthStage = false;
         // first, send type 1 message (with empty domain, workstation..., but it still works)
-        m_forceDisconnect = true;
-        KNTLM::getNegotiate(buf);
+        if (!KNTLM::getNegotiate(buf)) {
+            kWarning(7113) << "Error while constructing Type 1 NTLM authentication request";
+            m_isError = true;
+            return;
+        }
     } else {
+        m_finalAuthStage = true;
         // we've (hopefully) received a valid type 2 message: send type 3 message as last step
-        QString domain;
-        QString user = m_username;
-        if (user.contains('\\')) {
-            domain = user.section('\\', 0, 0);
-            user = user.section('\\', 1);
+        QString user, domain;
+        if (m_username.contains(QLatin1Char('\\'))) {
+            domain = m_username.section(QLatin1Char('\\'), 0, 0);
+            user = m_username.section(QLatin1Char('\\'), 1);
+        } else {
+            user = m_username;
         }
 
         m_forceKeepAlive = true;
-        QByteArray challenge;
-        KCodecs::base64Decode(m_challenge[0], challenge);
-        KNTLM::getAuth(buf, challenge, user, password, domain, QHostInfo::localHostName());
+        const QByteArray challenge = QByteArray::fromBase64(m_challenge[0]);
+
+        KNTLM::AuthFlags flags = KNTLM::Add_LM;
+        if (!m_config || !m_config->readEntry("EnableNTLMv2Auth", false)) {
+            flags |= KNTLM::Force_V1;
+        }
+
+        if (!KNTLM::getAuth(buf, challenge, user, m_password, domain, QLatin1String("WORKSTATION"), flags)) {
+            kWarning(7113) << "Error while constructing Type 3 NTLM authentication request";
+            m_isError = true;
+            return;
+        }
     }
 
     m_headerFragment = "NTLM ";
-    m_headerFragment += KCodecs::base64Encode(buf);
+    m_headerFragment += buf.toBase64();
     m_headerFragment += "\r\n";
 
     return;
@@ -637,7 +825,7 @@ void KHttpNegotiateAuthentication::fillKioAuthInfo(KIO::AuthInfo *ai) const
 {
     authInfoBoilerplate(ai);
     //### does GSSAPI supply anything realm-like? dummy value for now.
-    ai->realmValue = "Negotiate";
+    ai->realmValue = QLatin1String("Negotiate");
 }
 
 
@@ -698,7 +886,12 @@ void KHttpNegotiateAuthentication::generateResponse(const QString &user, const Q
         return;
     }
 
-    OM_uint32 req_flags = 0;
+    OM_uint32 req_flags;
+    if (m_config && m_config->readEntry("DelegateCredentialsOn", false))
+       req_flags = GSS_C_DELEG_FLAG;
+    else
+       req_flags = 0;
+
     // GSSAPI knows how to get the credentials its own way, so don't ask for any
     major_status = gss_init_sec_context(&minor_status, GSS_C_NO_CREDENTIAL,
                                         &ctx, server, mech_oid,
@@ -719,7 +912,7 @@ void KHttpNegotiateAuthentication::generateResponse(const QString &user, const Q
     }
 
     m_headerFragment = "Negotiate ";
-    m_headerFragment += QByteArray::fromRawData((const char *)output_token.value,
+    m_headerFragment += QByteArray::fromRawData(static_cast<const char *>(output_token.value),
                                                 output_token.length).toBase64();
     m_headerFragment += "\r\n";
 
