@@ -17,7 +17,6 @@
 #include "object.h"
 #include "JSVariableObject.h"
 #include "Parser.h"
-#include "value2string.h"
 
 #include <QtCore/QMap>
 #include <QtCore/QDebug>
@@ -36,10 +35,42 @@ QString KJS::UString::qstring() const
   return QString((QChar*) data(), size());
 }
 
+// from khtml/ecma/debugger/value2string.cpp
+QString valueToString(KJS::JSValue* value)
+{
+    switch(value->type())
+    {
+        case KJS::NumberType:
+        {
+            double v = 0.0;
+            value->getNumber(v);
+            return QString::number(v);
+        }
+        case KJS::BooleanType:
+            return value->getBoolean() ? "true" : "false";
+        case KJS::StringType:
+        {
+            KJS::UString s;
+            value->getString(s);
+            return '"' + s.qstring() + '"';
+        }
+        case KJS::UndefinedType:
+            return "undefined";
+        case KJS::NullType:
+            return "null";
+        case KJS::ObjectType:
+            return "[object " + static_cast<KJS::JSObject*>(value)->className().qstring() +"]";
+        case KJS::GetterSetterType:
+        case KJS::UnspecifiedType:
+        default:
+            return QString();
+    }
+}
+
 // from khtml/ecma/debugger/debugwindow.cpp
 static QString exceptionToString(KJS::ExecState* exec, KJS::JSValue* exceptionObj)
 {
-    QString exceptionMsg = KJSDebugger::valueToString(exceptionObj);
+    QString exceptionMsg = valueToString(exceptionObj);
 
     // Since we purposefully bypass toString, we need to figure out
     // string serialization ourselves.
@@ -158,6 +189,8 @@ void ECMAscriptTest::initTestCase()
                                                .filter( QRegExp( "^[^#].*" ) );
         }
     }
+
+    m_passed = 0;
 }
 
 static QByteArray getTextProperty( const QByteArray &property, const QByteArray &code )
@@ -177,6 +210,18 @@ static QByteArray getTextProperty( const QByteArray &property, const QByteArray
     return code.mid( from, to - from ).replace( "\\", "\\\\" ).replace( "\"", "\\\"" );
 }
 
+#define ECMATEST_VERIFY( expr ) \
+    do { \
+        const bool tmp_result = ( expr ); \
+        if ( tmp_result ) \
+            m_passed++; \
+        else \
+            m_failed++; \
+        if ( knownBroken ) \
+            QEXPECT_FAIL(QTest::currentDataTag(), "It is known that KJS doesn't pass this test", Abort); \
+        QVERIFY( tmp_result ); \
+    } while (0)
+
 static QMap< QByteArray, QByteArray > skips;
 
 void ECMAscriptTest::runAllTests()
@@ -239,38 +284,24 @@ void ECMAscriptTest::runAllTests()
     const bool knownBroken = expectedBroken.contains( QString::fromAscii( QTest::currentDataTag() ) );
 
     if ( expectedError.isEmpty() ) {
-        if ( knownBroken ) {
-            QWARN( "It is known that KJS doesn't pass this test" );
-            QVERIFY2( completion.complType() == KJS::Throw, "test expected to be broken now works!" );
-        } else {
-            QVERIFY( completion.complType() != KJS::Throw );
-        }
+        ECMATEST_VERIFY( completion.complType() != KJS::Throw );
     } else {
         if ( knownBroken && completion.complType() != KJS::Throw ) {
-            QWARN( "It is known that KJS doesn't pass this test" );
+            QEXPECT_FAIL(QTest::currentDataTag(), "It is known that KJS doesn't pass this test", Abort);
+            m_failed++;
+        }
+
+        QCOMPARE( completion.complType(), KJS::Throw );
+        QVERIFY( completion.value() != NULL );
+
+        const QString eMsg = exceptionToString( interp->execState(), completion.value() );
+
+        if ( expectedError == "^((?!NotEarlyError).)*$" ) {
+            ECMATEST_VERIFY( eMsg.indexOf( "NotEarlyError" ) == -1 );
+        } else if ( expectedError == "." ) {
+            // means "every exception passes
         } else {
-            QVERIFY( completion.complType() == KJS::Throw );
-            QVERIFY( completion.value() != NULL );
-
-            const QString eMsg = exceptionToString( interp->execState(), completion.value() );
-
-            if ( expectedError == "^((?!NotEarlyError).)*$" ) {
-                if ( knownBroken ) {
-                    QWARN( "It is known that KJS doesn't pass this test" );
-                    QVERIFY2( eMsg.indexOf( "NotEarlyError" ) >= 0, "test expected to be broken now works!" );
-                } else {
-                    QVERIFY( eMsg.indexOf( "NotEarlyError" ) == -1 );
-                }
-            } else if ( expectedError == "." ) {
-                // means "every exception passes
-            } else {
-                if ( knownBroken ) {
-                    QWARN( "It is known that KJS doesn't pass this test" );
-                    QVERIFY2( eMsg.indexOf( expectedError ) == -1, "test expected to be broken now works!" );
-                } else {
-                    QVERIFY( eMsg.indexOf( expectedError ) >= 0 );
-                }
-            }
+            ECMATEST_VERIFY( eMsg.indexOf( expectedError ) >= 0 );
         }
     }
 }
@@ -289,7 +320,7 @@ void ECMAscriptTest::runAllTests_data()
         QWARN( "===> Testing chapter " + chapter.toAscii() );
 
     // some tests fail when the suite is run as a whole
-    if ( chapter.isEmpty() || chapter == "ch15" ) {
+    if ( chapter.isEmpty() || chapter.startsWith("ch15") ) {
         const QByteArray endlessLoop = "this test causes an endless loop, avoid it for the moment";
         skips[ "S15.1.2.3_A6" ] = endlessLoop;
         skips[ "S15.1.3.1_A2.5_T1" ] = endlessLoop;
@@ -320,4 +351,9 @@ void ECMAscriptTest::cleanup()
     global->clearProperties();
 }
 
+void ECMAscriptTest::cleanupTestCase()
+{
+    qDebug() << "passed testcases:" << m_passed << "failed testcases:" << m_failed;
+}
+
 #include "ecmatest.moc"
