@@ -304,8 +304,11 @@ BrowserRun::NonEmbeddableResult BrowserRun::handleNonEmbeddable(const QString& _
                          this, SLOT(slotCopyToTempFileResult(KJob*)) );
                 return Delayed; // We'll continue after the job has finished
             }
-            if (selectedService)
+            if (selectedService) {
                 *selectedService = question.selectedService();
+                // KRun will use this when starting an app
+                KRun::setPreferredService(question.selectedService()->desktopEntryName());
+            }
         }
     }
 
