@@ -19,19 +19,26 @@
 
 #include "packagestructure.h"
 
+#include "config-plasma.h"
+
 #include <QDir>
 #include <QMap>
+#include <QMutableListIterator>
 #include <QFileInfo>
 
 #include <kconfiggroup.h>
+#include <kdebug.h>
+#ifndef PLASMA_NO_KIO
+#include <kio/job.h>
+#endif
+#include <kmimetype.h>
 #include <kstandarddirs.h>
 #include <kservicetypetrader.h>
-#include <kurl.h>
+#include <ktar.h>
 #include <ktemporaryfile.h>
 #include <ktempdir.h>
+#include <kurl.h>
 #include <kzip.h>
-#include <kio/netaccess.h>
-#include <kio/job.h>
 
 #include "package.h"
 #include "private/packages_p.h"
@@ -51,14 +58,14 @@ class ContentStructure
 
         ContentStructure(const ContentStructure &other)
         {
-            path = other.path;
+            paths = other.paths;
             name = other.name;
             mimetypes = other.mimetypes;
             directory = other.directory;
             required = other.required;
         }
 
-        QString path;
+        QStringList paths;
         QString name;
         QStringList mimetypes;
         bool directory : 1;
@@ -68,10 +75,14 @@ class ContentStructure
 class PackageStructurePrivate
 {
 public:
-    PackageStructurePrivate()
-        : metadata(0),
+    PackageStructurePrivate(const QString &t)
+        : type(t),
+          packageRoot("plasma/plasmoids"),
+          servicePrefix("plasma-applet-"),
+          metadata(0),
           externalPaths(false)
     {
+        contentsPrefixPaths << "contents/";
     }
 
     ~PackageStructurePrivate()
@@ -80,12 +91,11 @@ public:
     }
 
     void createPackageMetadata(const QString &path);
-
-    static QHash<QString, PackageStructure::Ptr> structures;
+    QStringList entryList(const QString &prefix, const QString &requestedPath);
 
     QString type;
     QString path;
-    QString contentsPrefix;
+    QStringList contentsPrefixPaths;
     QString packageRoot;
     QString servicePrefix;
     QMap<QByteArray, ContentStructure> contents;
@@ -94,16 +104,10 @@ public:
     bool externalPaths;
  };
 
-QHash<QString, PackageStructure::Ptr> PackageStructurePrivate::structures;
-
 PackageStructure::PackageStructure(QObject *parent, const QString &type)
     : QObject(parent),
-      d(new PackageStructurePrivate)
+      d(new PackageStructurePrivate(type))
 {
-    d->type = type;
-    d->contentsPrefix = "contents/";
-    d->packageRoot = "plasma/plasmoids/";
-    d->servicePrefix = "plasma-applet-";
 }
 
 PackageStructure::~PackageStructure()
@@ -117,11 +121,7 @@ PackageStructure::Ptr PackageStructure::load(const QString &packageFormat)
         return Ptr(new PackageStructure());
     }
 
-    PackageStructure::Ptr structure = PackageStructurePrivate::structures[packageFormat];
-
-    if (structure) {
-        return structure;
-    }
+    PackageStructure::Ptr structure;
 
     if (packageFormat == "Plasma/Applet") {
         structure = defaultPackageStructure(AppletComponent);
@@ -138,10 +138,13 @@ PackageStructure::Ptr PackageStructure::load(const QString &packageFormat)
     } else if (packageFormat == "Plasma/Theme") {
         structure = Theme::packageStructure();
         structure->d->type = "Plasma/Theme";
+    } else if (packageFormat == "Plasma/Generic") {
+        structure = defaultPackageStructure(GenericComponent);
+        structure->d->type = "Plasma/Generic";
+        structure->setDefaultPackageRoot(KStandardDirs::locate("data", "plasma/packages/"));
     }
 
     if (structure) {
-        PackageStructurePrivate::structures[packageFormat] = structure;
         return structure;
     }
 
@@ -172,17 +175,17 @@ PackageStructure::Ptr PackageStructure::load(const QString &packageFormat)
     if (!configPath.isEmpty()) {
         KConfig config(configPath);
         structure->read(&config);
-        PackageStructurePrivate::structures[packageFormat] = structure;
         return structure;
     }
 
     // try to load from absolute file path
     KUrl url(packageFormat);
     if (url.isLocalFile()) {
-        KConfig config(KIO::NetAccess::mostLocalUrl(url, NULL).toLocalFile(), KConfig::SimpleConfig);
+        KConfig config(url.toLocalFile(), KConfig::SimpleConfig);
         structure->read(&config);
-        PackageStructurePrivate::structures[structure->type()] = structure;
-    } else {
+    }
+#ifndef PLASMA_NO_KIO
+    else {
         KTemporaryFile tmp;
         if (tmp.open()) {
             KIO::Job *job = KIO::file_copy(url, KUrl(tmp.fileName()),
@@ -190,10 +193,10 @@ PackageStructure::Ptr PackageStructure::load(const QString &packageFormat)
             if (job->exec()) {
                 KConfig config(tmp.fileName(), KConfig::SimpleConfig);
                 structure->read(&config);
-                PackageStructurePrivate::structures[structure->type()] = structure;
             }
         }
     }
+#endif
 
     return structure;
 }
@@ -274,19 +277,32 @@ QStringList PackageStructure::entryList(const char *key)
         return QStringList();
     }
 
-    QDir dir(d->path + d->contentsPrefix + p);
-
-    if (dir.exists()) {
-        if (d->externalPaths) {
-            return dir.entryList(QDir::Files | QDir::Readable);
+    QStringList list;
+    if (d->contentsPrefixPaths.isEmpty()) {
+        // no prefixes is the same as d->contentsPrefixPths with QStringList() << QString()
+        list << d->entryList(QString(), p);
+    } else {
+        foreach (QString prefix, d->contentsPrefixPaths) {
+            list << d->entryList(prefix, p);
         }
+    }
 
-        // ensure that we don't return files outside of our base path
-        // due to symlink or ../ games
-        QString canonicalized = dir.canonicalPath();
-        if (canonicalized.startsWith(d->path)) {
-            return dir.entryList(QDir::Files | QDir::Readable);
-        }
+    return list;
+}
+
+QStringList PackageStructurePrivate::entryList(const QString &prefix, const QString &requestedPath)
+{
+    QDir dir(path + prefix + requestedPath);
+
+    if (externalPaths) {
+        return dir.entryList(QDir::Files | QDir::Readable);
+    }
+
+    // ensure that we don't return files outside of our base path
+    // due to symlink or ../ games
+    QString canonicalized = dir.canonicalPath();
+    if (canonicalized.startsWith(path)) {
+        return dir.entryList(QDir::Files | QDir::Readable);
     }
 
     return QStringList();
@@ -296,8 +312,16 @@ void PackageStructure::addDirectoryDefinition(const char *key,
                                               const QString &path, const QString &name)
 {
     ContentStructure s;
-    s.name = name;
-    s.path = path;
+
+    if (d->contents.contains(key)) {
+        s = d->contents[key];
+    }
+
+    if (!name.isEmpty()) {
+        s.name = name;
+    }
+
+    s.paths.append(path);
     s.directory = true;
 
     d->contents[key] = s;
@@ -306,13 +330,26 @@ void PackageStructure::addDirectoryDefinition(const char *key,
 void PackageStructure::addFileDefinition(const char *key, const QString &path, const QString &name)
 {
     ContentStructure s;
-    s.name = name;
-    s.path = path;
+
+    if (d->contents.contains(key)) {
+        s = d->contents[key];
+    }
+
+    if (!name.isEmpty()) {
+        s.name = name;
+    }
+
+    s.paths.append(path);
     s.directory = false;
 
     d->contents[key] = s;
 }
 
+void PackageStructure::removeDefinition(const char *key)
+{
+    d->contents.remove(key);
+}
+
 QString PackageStructure::path(const char *key) const
 {
     //kDebug() << "looking for" << key;
@@ -321,8 +358,20 @@ QString PackageStructure::path(const char *key) const
         return QString();
     }
 
-    //kDebug() << "found" << key << "and the value is" << it.value().path;
-    return it.value().path;
+    //kDebug() << "found" << key << "and the value is" << it.value().paths.first();
+    return it.value().paths.first();
+}
+
+QStringList PackageStructure::searchPath(const char *key) const
+{
+    //kDebug() << "looking for" << key;
+    QMap<QByteArray, ContentStructure>::const_iterator it = d->contents.constFind(key);
+    if (it == d->contents.constEnd()) {
+        return QStringList();
+    }
+
+    //kDebug() << "found" << key << "and the value is" << it.value().paths;
+    return it.value().paths;
 }
 
 QString PackageStructure::name(const char *key) const
@@ -393,8 +442,8 @@ void PackageStructure::setPath(const QString &path)
 
     if (valid) {
         QFileInfo info(basePath);
-        if (info.isDir()) {
-            basePath.append(QDir::separator());
+        if (info.isDir() && !basePath.endsWith('/')) {
+            basePath.append('/');
         }
         //kDebug() << "basePath is" << basePath;
     } else {
@@ -426,12 +475,16 @@ void PackageStructure::read(const KConfigBase *config)
 {
     d->contents.clear();
     d->mimetypes.clear();
-    d->type = config->group("").readEntry("Type", QString());
+    KConfigGroup general(config, QString());
+    d->type = general.readEntry("Type", QString());
+    d->contentsPrefixPaths = general.readEntry("ContentsPrefixPaths", d->contentsPrefixPaths);
+    d->packageRoot = general.readEntry("DefaultPackageRoot", d->packageRoot);
+    d->externalPaths = general.readEntry("AllowExternalPaths", d->externalPaths);
 
     QStringList groups = config->groupList();
     foreach (const QString &group, groups) {
+        KConfigGroup entry(config, group);
         QByteArray key = group.toAscii();
-        KConfigGroup entry = config->group(group);
 
         QString path = entry.readEntry("Path", QString());
         QString name = entry.readEntry("Name", QString());
@@ -452,12 +505,16 @@ void PackageStructure::read(const KConfigBase *config)
 
 void PackageStructure::write(KConfigBase *config) const
 {
-    config->group("").writeEntry("Type", type());
+    KConfigGroup general = KConfigGroup(config, "");
+    general.writeEntry("Type", type());
+    general.writeEntry("ContentsPrefixPaths", d->contentsPrefixPaths);
+    general.writeEntry("DefaultPackageRoot", d->packageRoot);
+    general.writeEntry("AllowExternalPaths", d->externalPaths);
 
     QMap<QByteArray, ContentStructure>::const_iterator it = d->contents.constBegin();
     while (it != d->contents.constEnd()) {
         KConfigGroup group = config->group(it.key());
-        group.writeEntry("Path", it.value().path);
+        group.writeEntry("Path", it.value().paths);
         group.writeEntry("Name", it.value().name);
         if (!it.value().mimetypes.isEmpty()) {
             group.writeEntry("Mimetypes", it.value().mimetypes);
@@ -475,12 +532,34 @@ void PackageStructure::write(KConfigBase *config) const
 
 QString PackageStructure::contentsPrefix() const
 {
-    return d->contentsPrefix;
+    return d->contentsPrefixPaths.isEmpty() ? QString() : d->contentsPrefixPaths.first();
 }
 
 void PackageStructure::setContentsPrefix(const QString &prefix)
 {
-    d->contentsPrefix = prefix;
+    d->contentsPrefixPaths.clear();
+    d->contentsPrefixPaths << prefix;
+}
+
+QStringList PackageStructure::contentsPrefixPaths() const
+{
+    return d->contentsPrefixPaths;
+}
+
+void PackageStructure::setContentsPrefixPaths(const QStringList &prefixPaths)
+{
+    d->contentsPrefixPaths = prefixPaths;
+
+    // the code assumes that the prefixes have a trailing slash
+    // so let's make that true here
+    QMutableStringListIterator it(d->contentsPrefixPaths);
+    while (it.hasNext()) {
+        it.next();
+
+        if (!it.value().endsWith('/')) {
+            it.setValue(it.value() % '/');
+        }
+    }
 }
 
 bool PackageStructure::installPackage(const QString &package, const QString &packageRoot)
@@ -521,20 +600,19 @@ void PackageStructure::setServicePrefix(const QString &servicePrefix)
 
 void PackageStructurePrivate::createPackageMetadata(const QString &path)
 {
-    if (metadata) {
-        delete metadata;
-        metadata = 0;
-    }
+    delete metadata;
+    metadata = 0;
 
     QString metadataPath(path + "/metadata.desktop");
     if (!QFile::exists(metadataPath)) {
-        metadataPath.clear();
         kWarning() << "No metadata file in the package, expected it at:" << metadataPath;
+        metadataPath.clear();
     }
 
     metadata = new PackageMetadata(metadataPath);
 }
 
+//FIXME KDE5: should be const
 PackageMetadata PackageStructure::metadata()
 {
     if (!d->metadata && !d->path.isEmpty()) {
@@ -543,15 +621,28 @@ PackageMetadata PackageStructure::metadata()
         if (fileInfo.isDir()) {
             d->createPackageMetadata(d->path);
         } else if (fileInfo.exists()) {
-            KZip archive(d->path);
-            if (archive.open(QIODevice::ReadOnly)) {
-                const KArchiveDirectory *source = archive.directory();
+            KArchive *archive = 0;
+            KMimeType::Ptr mimetype = KMimeType::findByPath(d->path);
+
+            if (mimetype->is("application/zip")) {
+                archive = new KZip(d->path);
+            } else if (mimetype->is("application/x-compressed-tar") || mimetype->is("application/x-gzip") ||
+                       mimetype->is("application/x-tar")|| mimetype->is("application/x-bzip-compressed-tar")) {
+                archive = new KTar(d->path);
+            } else {
+                kWarning() << "Could not open package file, unsupported archive format:" << d->path << mimetype->name();
+            }
+
+            if (archive && archive->open(QIODevice::ReadOnly)) {
+                const KArchiveDirectory *source = archive->directory();
                 KTempDir tempdir;
                 source->copyTo(tempdir.name());
                 d->createPackageMetadata(tempdir.name());
             } else {
                 kWarning() << "Could not open package file:" << d->path;
             }
+
+            delete archive;
         }
     }
 
